//
// C++ Implementation: yawp
//
// Description:
//
//
// Author: Ruan <ruans@kr8.co.za>, (C) 2008
//
// Copyright: Do what you want, whenever you want.
//
//
#ifndef YAWP_H
#define YAWP_H


#include <Plasma/Applet>
#include <Plasma/Svg>
#include <QGraphicsSceneMouseEvent>

#define MAX_FORECAST_DAYS       10

class YaWPConfigDialog;
class QTimer;
class QTimeLine;
class QLabel;
class QGraphicsSceneResizeEvent;
class KConfigDialog;
namespace Plasma { class Dialog; }

#include "yawpwserv.h"

//-----------------------------------------------------------------------------
/** Main plasmoid class.
@author Ruan <ruans@kr8.co.za>
*/
class YaWP : public Plasma::Applet
{
  Q_OBJECT
  public:

    YaWP(QObject *parent, const QVariantList &args);

    virtual ~YaWP();

    void paintInterface(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect& contentsRect);
    void init();

    void loadConfig();
    void saveConfig();

    void constraintsEvent(Plasma::Constraints constraints);

    QList<QAction *> contextualActions () { return m_actions; }

  protected:

    void paintPanel(QPainter *painter, const QStyleOptionGraphicsItem *option, const QRect& contentsRect);

    void createConfigurationInterface( KConfigDialog* );
    void mousePressEvent( QGraphicsSceneMouseEvent* );
    void loadCustomTheme();
    void mouseDoubleClickEvent( QGraphicsSceneMouseEvent* );

  protected slots:
    void timeout();
    void settingsChanged();
    void updateMessage(QString mess);
    void updateInfo(QObject *, int);

    void animationTimeout(int);
    void animationFinished();

  private:
    Plasma::Svg         m_svg;
    Plasma::Svg         m_customSvg;
    YaWPConfigDialog    *m_cdiag;
    QFont               m_font;
    QTimer              *m_timer;
    QString             m_message;
    int                 m_interval;
    bool                m_inPanel;
    bool                m_panelHorizontal;
    int                 m_panelItemCount;
    float               m_panelElementSize;
    float               m_panelElementBorderSize;
    QSizeF              m_expandedArea;
    QSizeF              m_origExpandedArea;
    int                 m_panelLayoutSwitched; // plasma resize event counter.

    QString             m_theme;
    QString             m_city;
    QString             m_country;
    QString             m_location;
    QString             m_service;
    int                 m_units;
    bool                m_useCustomTheme;
    QString             m_customTheme;
    bool                m_useCustomThemeBackground;
    Plasma::Dialog      *m_dialog;
    QLabel              *m_dialogLabel;
    bool                m_panelActualTemp;
    bool                m_panelActualIcon;
    bool                m_panelActualBoth;
    bool                m_panelForecastTemp;
    bool                m_panelForecastIcon;
    bool                m_panelForecastBoth;
    int                 m_panelForecastDays;

    YaWPWeatherInfo     m_info;

    YaWPWeatherService * m_wservice;
    YaWPWeatherService * m_googlewservice;
    YaWPWeatherService * m_accuweatherwservice;

    QList<QAction *> m_actions; /**< Contextual actions */

    float       m_scale;
    QTimeLine * m_animation;
    int         m_animationWhat;
    bool        m_animationActive;
    QRectF      m_wrect;

    enum { DAYS, EMBLEM, INFO, SATELLITE };

    int         m_whichEmblem;
    float       m_emblemSize;
    int         m_emblem[MAX_FORECAST_DAYS];
    float       m_animationTickEmblem[MAX_FORECAST_DAYS];
    enum { EMBLEM_DAY, EMBLEM_NIGHT };

    int         m_days;
    int         m_animationTickDays;
    enum { DAYS_NAMES, DAYS_NUMBERS };

    int         m_infoIcon;
    int         m_animationTickInfoIcon;
    int         m_infoIconDay;
    enum { INFO_CLOSED, INFO_OPEN };

    int         m_satellite;
    int         m_animationTickSatellite;
    enum { SATELLITE_CLOSED, SATELLITE_OPEN };

    bool        m_resized;

    QString veryShortDate();

    void paintEmblem( QPainter *, int, const YaWPDay *, float, float, float, bool = true);
    void paintDegrees( QPainter *, int, const YaWPDay *, QFont, float );
    void paintDayNames( QPainter *, QFont, QRectF, float );
    void paintInformation( QPainter *, QFont );
    void paintSatellite( QPainter * );

    int  clickOnEmblem( QPointF );
    int  clickOnPanelEmblem( QPointF );
    bool clickOnDays( QPointF );
    bool clickOnInfoIcon( QPointF );
    bool clickOnInfoIconLeft( QPointF );
    bool clickOnInfoIconRight( QPointF );
    bool clickOnSatelliteIcon( QPointF );

    void updatePanelSize(); // update panel size after config changed
    bool isDayTime(QTime currentTime, QTime sunriseTime, QTime sunsetTime); // return true if now is night
};


#endif
