//
// C++ Implementation: yawp
//
// Description:
//
//
// Author: Ruan <ruans@kr8.co.za>, (C) 2008
//
// Copyright: Do what you want, whenever you want.
//
//
#ifndef YAWP_WSERV_H
#define YAWP_WSERV_H

#include "yawpwday.h"

#include <QObject>
#include <kdebug.h>
#include <kio/job.h>

//-----------------------------------------------------------------------------
/** The abstract base class of a weather service.
	@see YaWPGoogleWeatherService
	@author Ruan <ruans@kr8.co.za>
 */
class YaWPWeatherService : public QObject {
     Q_OBJECT
  public:
    static const int SUCCESS = 0;
    static const int CONNECTION_FAILED = 1;
    static const int DATA_FAILED = 2;
    static const int ERROR = 3;
    static const int CACHE = 4;

    virtual ~YaWPWeatherService(){}

    virtual void update(const QMap<QString,QString> &data);

    const QString & serviceName() const { return m_name; }
    const YaWPWeatherInfo& info(){return m_info;}
    const QPixmap& image() { return m_image; }
    const QPixmap& logo() { return m_logo; }
    
    // Static so we can use it in YaWPConfigDialog :-)
    static QString toPercentEncoding(const QString &);
  signals:
    void completed(QObject *, int);

  protected:
    YaWPWeatherService(const QString &name, int dayCount);

    bool readIconMapping(const QString & resource);
    QString mapIcon(const QString &icon) const;

    virtual bool parseInfo(const QByteArray &data, YaWPWeatherInfo &info) = 0;

    QMap<QString,QString> m_icons;
    YaWPWeatherInfo m_info;
    QPixmap m_image;
    QPixmap m_logo;
    QString m_xmlurl;
    QString m_imageurl;
    QString m_dir;
    int m_code;
    
  protected slots:
    void slotXmlData(KIO::Job *, const QByteArray &);
    void slotXmlResult(KJob *);
    void slotImageData(KIO::Job *, const QByteArray &);
    void slotImageResult(KJob *);

  private:
    QString m_name;
    QByteArray m_xmldata;
    QByteArray m_imagedata;

    static const int UNDEFINED = -1;    
};

//-----------------------------------------------------------------------------
/** An google implementation of YaWPWeatherService.
	@author Ruan <ruans@kr8.co.za>
 */
class YaWPGoogleWeatherService : public YaWPWeatherService {
    Q_OBJECT
  public:
    YaWPGoogleWeatherService();

    virtual ~YaWPGoogleWeatherService();

    void update(const QMap<QString,QString> &data);

  private:
    bool parseInfo(const QByteArray &data, YaWPWeatherInfo &info);

    QString             m_city;
    QString             m_country;
};

//-----------------------------------------------------------------------------
/** An google implementation of YaWPWeatherService.
@author Ruan <ruans@kr8.co.za>
*/
class YaWPForecastfoxWeatherService : public YaWPWeatherService {
    Q_OBJECT
  public:
    YaWPForecastfoxWeatherService();

    virtual ~YaWPForecastfoxWeatherService();

    void update(const QMap<QString,QString> &data);

  private:
    bool parseInfo(const QByteArray &data, YaWPWeatherInfo &info);

    QString             m_location;
};


#endif //YAWP_WSERV_H
